==> musashi-core-1.0.3.tar.gz: musashi-core-1.0.3/cmd/xt2arff.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 1.0 : (2003/12/02)                                                 */
/* 1.1 : category°ͥꥹȤΥޤ˴ؤХ(2003/12/03)           */
/* 1.2 : category°Ǹ˽Ϥ褦ѹ(2004/02/05)                   */
/*============================================================================*/
#include <musashi.h>
#include <stdlib.h>
#include <string.h>

#include <xt2arffHelp.h>
struct mssComHelp comHelp={
  "xt2arff",      /* ޥ̾       */
  "1.2",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*            */                                      
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */                                      
  HELPH           /* ۡڡ     */
};
    
extern struct mssGlobalVariables mssGV;

int main(int argc, char *argv[]){
/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/* ͹                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLD optNUM={
    OFLD,   /* ץ󥿥                                             */
    "n",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    1,      /* ɽĤ뤫ɤ(0:Բ,1:)                      */
    0,      /* ̾Ǥ뤫ɤ(0:Բ,1:)                    */
    NULL,   /* ܥץ(%ʲ)ǻǽʸ                        */
            /* ex) ԲĤξNULL, "nr": "-f ̾%rn"λǽ     */
    NUMT,   /* ΥץΥȥ(Helpɽ)                         */
    NUMC,   /* ΥץΥ(Helpɽ)                         */
    NUMF    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
  };

/*----------------------------------------------------------------------------*/
/* ƥ                                                               */
/*----------------------------------------------------------------------------*/
  MssOptFLD optCAT={
    OFLD,   /* ץ󥿥                                             */
    "d",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    1,      /* ɽĤ뤫ɤ(0:Բ,1:)                      */
    0,      /* ̾Ǥ뤫ɤ(0:Բ,1:)                    */
    NULL,   /* ܥץ(%ʲ)ǻǽʸ                        */
            /* ex) ԲĤξNULL, "nr": "-f ̾%rn"λǽ     */
    CATT,   /* ΥץΥȥ(Helpɽ)                         */
    CATC,   /* ΥץΥ(Helpɽ)                         */
    CATF    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptOTF optOTF={
    OOTF,   /* ץ󥿥                                             */
    "o",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    OTFT,   /* ΥץΥȥ(Helpɽ)                         */
    OTFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̽                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLG optZIP={
    OFLG,   /* ץ󥿥                                             */
    "z",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    ZIPT,   /* ΥץΥȥ(Helpɽ)                         */
    ZIPC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
  void *opt[]={&optNUM,&optCAT,&optINF,&optOTF,&optZIP,NULL};
    
/*============================================================================*/
/* ѿ                                                             */
/*============================================================================*/
  struct mssHeader *hdi; /*ϥե<head>Ǽ¤*/

  struct mssFPR    *fpr; /*ϥե빽¤*/
  struct mssFPW    *fpw; /*ϥե빽¤*/
  struct mssFldRec *fr;  /*-ԥХåե¤*/

  struct mssHash **hash=NULL;  /* ƥ°ͤǼhash */
  struct mssHashNode *hn; /* 嵭hashΥΡɤ򼨤ѿ */
  MssValue v; /* hashϿѥߡ */
  int fldNo;
  int firstFlg;
  char *str;
  int i,j;

/*------------------------------------------------------------------------------
ѥ᡼
------------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);        /* ʥʤɤν     */
  mssHelpDoc(opt,&comHelp,argc,argv); /* إ                       */
  mssSetOption(opt,argc,argv);        /* ޥɥץ     */
  fpr=mssOpenFPR(optINF.str,4);       /* ϥե륪ץ         */
  hdi=mssReadHeader(fpr);             /* إåɤ߹             */
  mssSetOptFld(&optNUM, hdi);        /* -f ܤإåܤ˴ϢŤ */
  mssSetOptFld(&optCAT, hdi);        /* -f ܤإåܤ˴ϢŤ */

  /*°ʤȤĤϻꤵƤ뤫*/
  if(!optCAT.set && !optNUM.set){
    mssShowErrMsg("must be specify a parameter -n or -d.");
    exit(mssErrorNoDefault);
  }

/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  fr=mssInitFldRec(hdi->flds->cnt);

  /* ܤ٥󤷡ƥ°ͤϥå˳Ǽ */
  if(optCAT.set){

    hash=mssCalloc(sizeof(struct mssHash *)*optCAT.flds->cnt,"main");
    for(i=0;i<optCAT.flds->cnt;i++){
      *(hash+i)=mssInitHash(101); /* hash¤Τν */
    }

    while(EOF!=mssReadFldRec(fpr,fr)){
      /* ܿLoop */
      for(i=0;i<optCAT.flds->cnt;i++){
        fldNo=MssFlds2num(optCAT.flds,i);
        if(!MssIsNull(*(fr->pnt+fldNo))){
          /* ܤ(ʸ)ϥåɽϿ       */
          /* ϥåɽˤʤСΡɤɲä */
          mssHashInsert(*(hash+i),*(fr->pnt+fldNo),v);
        }
      }
    }
  }

/*----------------------------------------------------------------------------*/
/*arffإå                                                            */
/*----------------------------------------------------------------------------*/
  /*ɸϥץ+إåν*/
  fpw=mssOpenFPW(optOTF.str,optZIP.set,0);

  /* relation */
  mssWriteStr("@RELATION\t\"",fpw);
  if(hdi->title==NULL){
    mssWriteStr("By MUSASHI\"",fpw);
  }else{
    mssWriteStr(hdi->title,fpw);
    mssWriteStr("\"",fpw);
  } 
  mssWriteStr("\n\n",fpw);

  /* attribute(numeric) */
  if(optNUM.set){
    for(i=0;i<optNUM.flds->cnt;i++){
      mssWriteStr("@ATTRIBUTE\t",fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr(MssFlds2name(optNUM.flds,i),fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr("\t",fpw);
      mssWriteStr("NUMERIC\n",fpw);
    }
  }

  /* attribute(categorical) */
  if(optCAT.set){
    for(i=0;i<optCAT.flds->cnt;i++){
      mssWriteStr("@ATTRIBUTE\t",fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr(MssFlds2name(optCAT.flds,i),fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr("\t",fpw);
      mssWriteStr("{",fpw);
      firstFlg=1;
      for(j=0; j<(*(hash+i))->hashVal; j++){
        if( NULL != (hn=*((*(hash+i))->node+j)) ){
          while(hn!=NULL){
            if(firstFlg) firstFlg=0;
            else         mssWriteStr(",",fpw);
            mssWriteStr("\"",fpw);
            mssWriteStr(hn->str,fpw);
            mssWriteStr("\"",fpw);
            hn=hn->next;
          }
        }
      }
      mssWriteStr("}\n",fpw);
    }
  }
  mssWriteStr("\n",fpw);

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/
  mssSeekTopFPR(fpr);
  mssWriteStr("@DATA\n",fpw);
  while( EOF != mssReadFldRec(fpr,fr) ){
    mssGV.inCnt++;
    firstFlg=1;

    /*ܤν񤭽Ф(Numerical)*/
    if(optNUM.set){
      for(i=0; i<optNUM.flds->cnt; i++){
        if(firstFlg) firstFlg=0;
        else         mssWriteStr(",",fpw);
        str=*(fr->pnt+MssFlds2num(optNUM.flds,i));
        if( MssIsNull(str) ){
          mssWriteStr("?",fpw);
        }else{
          mssWriteStr(*(fr->pnt+MssFlds2num(optNUM.flds,i)),fpw);
        }
      }
    }

    /*ܤν񤭽Ф(Categorical)*/
    if(optCAT.set){
      for(i=0; i<optCAT.flds->cnt; i++){
        if(firstFlg) firstFlg=0;
        else         mssWriteStr(",",fpw);
        str=*(fr->pnt+MssFlds2num(optCAT.flds,i));
        if( MssIsNull(str) ){
          mssWriteStr("?",fpw);
        }else{
          mssWriteStr("\"",fpw);
          mssWriteStr(*(fr->pnt+MssFlds2num(optCAT.flds,i)),fpw);
          mssWriteStr("\"",fpw);
        }
      }
    }

    mssWriteRet(fpw);
    mssGV.outCnt++;
  }
  mssFreeFldRec(fr);

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssCloseFPR(fpr);        /*ϥեΥ*/
  mssCloseFPW(fpw);        /*ϥեΥ*/
  mssFreeHeader(hdi);      /*ϥإåΰ賫*/
  mssFreeOption(opt);      /*ץΰ賫*/
  mssShowEndMsg();         /*λå*/
  mssEnd(mssExitSuccess);  /*λ*/
  return(0); /* to avoid warning message*/
}

==> musashi-core-1.0.4-pre6.tar.gz: musashi-core-1.0.4-pre6/cmd/xt2arff.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 1.0 : (2003/12/02)                                                 */
/* 1.1 : category°ͥꥹȤΥޤ˴ؤХ(2003/12/03)           */
/* 1.2 : category°Ǹ˽Ϥ褦ѹ(2004/02/05)                   */
/* 1.3 : -i ѥ᡼ץˤʤäƤХ(2004/04/19)            */
/* 1.4 : ʸ󷿡շб(2004/07/31)                                   */
/*============================================================================*/
#include <musashi.h>
#include <stdlib.h>
#include <string.h>

#include <xt2arffHelp.h>
struct mssComHelp comHelp={
  "xt2arff",      /* ޥ̾       */
  "1.4",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*            */                                      
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */                                      
  HELPH           /* ۡڡ     */
};
    
extern struct mssGlobalVariables mssGV;

int main(int argc, char *argv[]){
/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/* ͹                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLD optNUM={
    OFLD,   /* ץ󥿥                                             */
    "n",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    1,      /* ɽĤ뤫ɤ(0:Բ,1:)                      */
    0,      /* ̾Ǥ뤫ɤ(0:Բ,1:)                    */
    NULL,   /* ܥץ(%ʲ)ǻǽʸ                        */
            /* ex) ԲĤξNULL, "nr": "-f ̾%rn"λǽ     */
    NUMT,   /* ΥץΥȥ(Helpɽ)                         */
    NUMC,   /* ΥץΥ(Helpɽ)                         */
    NUMF    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
  };

/*----------------------------------------------------------------------------*/
/* ƥ                                                               */
/*----------------------------------------------------------------------------*/
  MssOptFLD optCAT={
    OFLD,   /* ץ󥿥                                             */
    "d",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    1,      /* ɽĤ뤫ɤ(0:Բ,1:)                      */
    0,      /* ̾Ǥ뤫ɤ(0:Բ,1:)                    */
    NULL,   /* ܥץ(%ʲ)ǻǽʸ                        */
            /* ex) ԲĤξNULL, "nr": "-f ̾%rn"λǽ     */
    CATT,   /* ΥץΥȥ(Helpɽ)                         */
    CATC,   /* ΥץΥ(Helpɽ)                         */
    CATF    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
  };

/*----------------------------------------------------------------------------*/
/* ʸ                                                                 */
/*----------------------------------------------------------------------------*/
  MssOptFLD optSTR={
    OFLD,   /* ץ󥿥                                             */
    "s",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    1,      /* ɽĤ뤫ɤ(0:Բ,1:)                      */
    0,      /* ̾Ǥ뤫ɤ(0:Բ,1:)                    */
    NULL,   /* ܥץ(%ʲ)ǻǽʸ                        */
            /* ex) ԲĤξNULL, "nr": "-f ̾%rn"λǽ     */
    STRT,   /* ΥץΥȥ(Helpɽ)                         */
    STRC,   /* ΥץΥ(Helpɽ)                         */
    STRF    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
  };


/*----------------------------------------------------------------------------*/
/* չ                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLD optDAT={
    OFLD,   /* ץ󥿥                                             */
    "D",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    0,      /* ɽĤ뤫ɤ(0:Բ,1:)                      */
    0,      /* ̾Ǥ뤫ɤ(0:Բ,1:)                    */
    "t",    /* ܥץ(%ʲ)ǻǽʸ                        */
            /* ex) ԲĤξNULL, "nr": "-f ̾%rn"λǽ     */
    DATT,   /* ΥץΥȥ(Helpɽ)                         */
    DATC,   /* ΥץΥ(Helpɽ)                         */
    DATF    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptOTF optOTF={
    OOTF,   /* ץ󥿥                                             */
    "o",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    OTFT,   /* ΥץΥȥ(Helpɽ)                         */
    OTFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̽                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLG optZIP={
    OFLG,   /* ץ󥿥                                             */
    "z",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    ZIPT,   /* ΥץΥȥ(Helpɽ)                         */
    ZIPC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
  void *opt[]={&optNUM,&optCAT,&optSTR,&optDAT,&optINF,&optOTF,&optZIP,NULL};
    
/*============================================================================*/
/* ѿ                                                             */
/*============================================================================*/
  struct mssHeader *hdi; /*ϥե<head>Ǽ¤*/

  struct mssFPR    *fpr; /*ϥե빽¤*/
  struct mssFPW    *fpw; /*ϥե빽¤*/
  struct mssFldRec *fr;  /*-ԥХåե¤*/

  struct mssHash **hash=NULL;  /* ƥ°ͤǼhash */
  struct mssHashNode *hn; /* 嵭hashΥΡɤ򼨤ѿ */
  MssValue v; /* hashϿѥߡ */
  int fldNo;
  int firstFlg;
  char *str;
  int i,j;
  int *withTime=NULL;
  int len;

/*------------------------------------------------------------------------------
ѥ᡼
------------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);        /* ʥʤɤν     */
  mssHelpDoc(opt,&comHelp,argc,argv); /* إ                       */
  mssSetOption(opt,argc,argv);        /* ޥɥץ     */
  fpr=mssOpenFPR(optINF.str,4);       /* ϥե륪ץ         */
  hdi=mssReadHeader(fpr);             /* إåɤ߹             */
  mssSetOptFld(&optNUM, hdi);        /* -n ܤإåܤ˴ϢŤ */
  mssSetOptFld(&optCAT, hdi);        /* -d ܤإåܤ˴ϢŤ */
  mssSetOptFld(&optSTR, hdi);        /* -s ܤإåܤ˴ϢŤ */
  mssSetOptFld(&optDAT, hdi);        /* -D ܤإåܤ˴ϢŤ */

  /*°ʤȤĤϻꤵƤ뤫*/
  if(!optCAT.set && !optNUM.set && !optSTR.set && !optDAT.set){
    mssShowErrMsg("must be specify at least one of parameters -n,-d,-s,-D.");
    exit(mssErrorNoDefault);
  }

/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  fr=mssInitFldRec(hdi->flds->cnt);

  /* ܤ٥󤷡ƥ°ͤϥå˳Ǽ */
  if(optCAT.set){

    hash=mssCalloc(sizeof(struct mssHash *)*optCAT.flds->cnt,"main");
    for(i=0;i<optCAT.flds->cnt;i++){
      *(hash+i)=mssInitHash(101); /* hash¤Τν */
    }

    while(EOF!=mssReadFldRec(fpr,fr)){
      /* ܿLoop */
      for(i=0;i<optCAT.flds->cnt;i++){
        fldNo=MssFlds2num(optCAT.flds,i);
        if(!MssIsNull(*(fr->pnt+fldNo))){
          /* ܤ(ʸ)ϥåɽϿ       */
          /* ϥåɽˤʤСΡɤɲä */
          mssHashInsert(*(hash+i),*(fr->pnt+fldNo),v);
        }
      }
    }
  }

/*----------------------------------------------------------------------------*/
/*arffإå                                                            */
/*----------------------------------------------------------------------------*/
  /*ɸϥץ+إåν*/
  fpw=mssOpenFPW(optOTF.str,optZIP.set,0);

  /* relation */
  mssWriteStr("@RELATION\t\"",fpw);
  if(hdi->title==NULL){
    mssWriteStr("By MUSASHI\"",fpw);
  }else{
    mssWriteStr(hdi->title,fpw);
    mssWriteStr("\"",fpw);
  } 
  mssWriteStr("\n\n",fpw);

  /* attribute(string) */
  if(optSTR.set){
    for(i=0;i<optSTR.flds->cnt;i++){
      mssWriteStr("@ATTRIBUTE\t",fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr(MssFlds2name(optSTR.flds,i),fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr("\t",fpw);
      mssWriteStr("string\n",fpw);
    }
  }

  /* attribute(date) */
  if(optDAT.set){
    withTime=mssMalloc(sizeof(int)*optDAT.flds->cnt,"xt2arff");
    for(i=0;i<optDAT.flds->cnt;i++){
      mssWriteStr("@ATTRIBUTE\t",fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr(MssFlds2name(optDAT.flds,i),fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr("\t",fpw);
      if(mssIsFldOptOn(&optDAT,i,'t')){
        mssWriteStr("date yyyyMMddHHmmss\n",fpw);
        *(withTime+i)=1;
      }else{
        mssWriteStr("date yyyyMMdd\n",fpw);
        *(withTime+i)=0;
      }
    }
  }


  /* attribute(numeric) */
  if(optNUM.set){
    for(i=0;i<optNUM.flds->cnt;i++){
      mssWriteStr("@ATTRIBUTE\t",fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr(MssFlds2name(optNUM.flds,i),fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr("\t",fpw);
      mssWriteStr("numeric\n",fpw);
    }
  }

  /* attribute(categorical) */
  if(optCAT.set){
    for(i=0;i<optCAT.flds->cnt;i++){
      mssWriteStr("@ATTRIBUTE\t",fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr(MssFlds2name(optCAT.flds,i),fpw);
      mssWriteStr("\"",fpw);
      mssWriteStr("\t",fpw);
      mssWriteStr("{",fpw);
      firstFlg=1;
      for(j=0; j<(*(hash+i))->hashVal; j++){
        if( NULL != (hn=*((*(hash+i))->node+j)) ){
          while(hn!=NULL){
            if(firstFlg) firstFlg=0;
            else         mssWriteStr(",",fpw);
            mssWriteStr("\"",fpw);
            mssWriteStr(hn->str,fpw);
            mssWriteStr("\"",fpw);
            hn=hn->next;
          }
        }
      }
      mssWriteStr("}\n",fpw);
    }
  }
  mssWriteStr("\n",fpw);

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/
  mssSeekTopFPR(fpr);
  mssWriteStr("@DATA\n",fpw);
  while( EOF != mssReadFldRec(fpr,fr) ){
    mssGV.inCnt++;
    firstFlg=1;

    /*ܤν񤭽Ф(String)*/
    if(optSTR.set){
      for(i=0; i<optSTR.flds->cnt; i++){
        if(firstFlg) firstFlg=0;
        else         mssWriteStr(",",fpw);
        str=*(fr->pnt+MssFlds2num(optSTR.flds,i));
        if( MssIsNull(str) ){
          mssWriteStr("?",fpw);
        }else{
          mssWriteStr(*(fr->pnt+MssFlds2num(optSTR.flds,i)),fpw);
        }
      }
    }

    /*ܤν񤭽Ф(Date)*/
    if(optDAT.set){
      for(i=0; i<optDAT.flds->cnt; i++){
        if(firstFlg) firstFlg=0;
        else         mssWriteStr(",",fpw);
        str=*(fr->pnt+MssFlds2num(optDAT.flds,i));
        len=strlen(str);
        if( MssIsNull(str) ){
          mssWriteStr("?",fpw);
        }else{
          if(*(withTime+i)){
            if(len==14) mssWriteStr(*(fr->pnt+MssFlds2num(optDAT.flds,i)),fpw);
            else        mssWriteStr("?",fpw);
          }else{
            if(len==8) mssWriteStr(*(fr->pnt+MssFlds2num(optDAT.flds,i)),fpw);
            else       mssWriteStr("?",fpw);
          }
        }
      }
    }

    /*ܤν񤭽Ф(Numerical)*/
    if(optNUM.set){
      for(i=0; i<optNUM.flds->cnt; i++){
        if(firstFlg) firstFlg=0;
        else         mssWriteStr(",",fpw);
        str=*(fr->pnt+MssFlds2num(optNUM.flds,i));
        if( MssIsNull(str) ){
          mssWriteStr("?",fpw);
        }else{
          mssWriteStr(*(fr->pnt+MssFlds2num(optNUM.flds,i)),fpw);
        }
      }
    }

    /*ܤν񤭽Ф(Categorical)*/
    if(optCAT.set){
      for(i=0; i<optCAT.flds->cnt; i++){
        if(firstFlg) firstFlg=0;
        else         mssWriteStr(",",fpw);
        str=*(fr->pnt+MssFlds2num(optCAT.flds,i));
        if( MssIsNull(str) ){
          mssWriteStr("?",fpw);
        }else{
          mssWriteStr("\"",fpw);
          mssWriteStr(*(fr->pnt+MssFlds2num(optCAT.flds,i)),fpw);
          mssWriteStr("\"",fpw);
        }
      }
    }

    mssWriteRet(fpw);
    mssGV.outCnt++;
  }
  mssFreeFldRec(fr);

  mssFree(withTime);

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssCloseFPR(fpr);        /*ϥեΥ*/
  mssCloseFPW(fpw);        /*ϥեΥ*/
  mssFreeHeader(hdi);      /*ϥإåΰ賫*/
  mssFreeOption(opt);      /*ץΰ賫*/
  mssShowEndMsg();         /*λå*/
  mssEnd(mssExitSuccess);  /*λ*/
  return(0); /* to avoid warning message*/
}
